@@ -13,17 +13,18 @@ allfuncs="`cd $sdir_top; echo ${allfuncs}`"
 # (1) the glob got expanded (2) we are not looking at directories.
 for file in $allfuncs; do
   if test -f $sdir_top/$file; then
-    if test x$FUNCTIONS_SUBDIRS != x -a x$FUNCTIONS_SUBDIRS != xno; then
-      case "$file" in
+    case "$file" in
       */CVS/*) continue;;
-      Completion/comp*)
-        subdir="`echo $file | sed -e 's%/[^/]*/[^/]*$%%'`"
-        instdir="$fndir/Completion"
-        ;;
-      Completion/*)
+    esac
+    if test x$FUNCTIONS_SUBDIRS != x && test x$FUNCTIONS_SUBDIRS != xno; then
+      case "$file" in
+      Completion/*/*)
         subdir="`echo $file | sed -e 's%/[^/]*/[^/]*$%%'`"
         instdir="$fndir/$subdir"
         ;;
+      Completion/*)
+        instdir="$fndir/Completion"
+        ;;
       *)
         subdir="`echo $file | sed -e 's%/[^/]*$%%' -e 's%^Functions/%%'`"
         instdir="$fndir/$subdir"
@@ -34,5 +35,8 @@ for file in $allfuncs; do
     fi
     test -d $instdir || /bin/sh $sdir_top/mkinstalldirs $instdir || exit 1
     $INSTALL_DATA $sdir_top/$file $instdir || exit 1
+    if test -x $sdir_top/$file; then
+	chmod +x $instdir/`echo $file | sed -e 's%^.*/%%'`
+    fi
   fi
 done
