@@ -485,16 +485,18 @@ set_killring(UNUSED(Param pm), char **x)
 	 */
 	int kpos = 0;
 	kringsize = arrlen(x);
-	kring = (Cutbuffer)zshcalloc(kringsize * sizeof(struct cutbuffer));
-	for (p = x; *p; p++) {
-	    int n, len = strlen(*p);
-	    kptr = kring + kpos;
-	    unmetafy(*p, &n);
-	    kptr->len = n;
-	    kptr->buf = (char *)zalloc(kptr->len);
-	    memcpy(kptr->buf, *p, kptr->len);
-	    zfree(*p, len+1);
-	    kpos = (kpos + kringsize -1 ) % kringsize;
+	if (kringsize != 0) {
+	    kring = (Cutbuffer)zshcalloc(kringsize * sizeof(struct cutbuffer));
+	    for (p = x; *p; p++) {
+	        int n, len = strlen(*p);
+	        kptr = kring + kpos;
+	        unmetafy(*p, &n);
+	        kptr->len = n;
+	        kptr->buf = (char *)zalloc(kptr->len);
+	        memcpy(kptr->buf, *p, kptr->len);
+	        zfree(*p, len+1);
+	        kpos = (kpos + kringsize -1 ) % kringsize;
+	    }
 	}
 	free(x);
     }
